SystemOrganization addCategory: #FuelPackageLoader!
SystemOrganization addCategory: #'FuelPackageLoader-Test'!

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 03:44'!
classVariableBindingForFix: aClassVariableAssociationInSource

	^classPool associationAt: aClassVariableAssociationInSource key! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 04:19'!
copyWithLocalProtocol

	| copy |
	copy := self copy.

	copy organization: organization deepCopy.
	copy environment: environment.
	copy basicCategory: category.	
		
	copy traitComposition: self traitComposition.
	copy classSide traitComposition: self classSide traitComposition.
	
	copy removeForeignProtocol.
	copy classSide removeForeignProtocol.
	
	copy resetClassVariables.
	copy fixClassVariablesInAll: self methodDictionary values copiedFrom: self classPool.
	copy fixClassVariablesInAll: self classSide methodDictionary values copiedFrom: self classPool.
	
	^copy! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 04:01'!
fixClassVariablesIn: aCompiledMethod copiedFrom: aClassVariableAssociationInSource

	| index | 
	index := aCompiledMethod indexOfLiteral: aClassVariableAssociationInSource.
	index > 0 
		ifTrue: [ 
			aCompiledMethod 
				literalAt: index 
				put: (self classVariableBindingForFix: aClassVariableAssociationInSource)]
	! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 04:00'!
fixClassVariablesInAll: compiledMethods copiedFrom: sourceClassPool

	compiledMethods do: [ :aCompiledMethod | 
		sourceClassPool associationsDo: [ :aClassVariableAssociationInSource | 
			self fixClassVariablesIn: aCompiledMethod copiedFrom: aClassVariableAssociationInSource ] ]
	! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 02:37'!
fuelAccept: aVisitor

	^(Smalltalk globals includes: self)
		ifTrue: [ aVisitor visitSubstitution: self ]
		ifFalse: [ aVisitor visitClass: self ]! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 02:53'!
fuelSubstitution

	^self copyWithLocalProtocol ! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 03:50'!
resetClassVariables

	self classPool keys do: [ :key | classPool at: key put: nil ]! !

!TClassAndTraitDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 00:27'!
removeCategorySilently: aCategory

	SystemChangeNotifier uniqueInstance doSilently: [ self removeCategory: aCategory ]! !

!ClassOrganizer methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:32'!
removeCategorySilently: cat 

	super removeCategory: cat! !

!ClassOrganizer methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:49'!
removeElementSilently: element

	^super removeElement: element! !

TestCase subclass: #FLPackageLoaderTest
	uses: FLTSerializationTest
	instanceVariableNames: 'streams classFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader-Test'!

FLPackageLoaderTest class
	uses: FLTSerializationTest classTrait
	instanceVariableNames: ''!

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
classFactory
	^ classFactory! !

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
classFactory: anObject
	classFactory := anObject! !

!FLPackageLoaderTest methodsFor: 'running' stamp: 'MartinDias 8/21/2011 21:11'!
storeRemoveAndLoadStubPackage

	| aStream package |
	aStream := self writeStream.
	FLPackageStore new storeOn: aStream packageNamed: classFactory packageName.
	
	classFactory cleanUp.
	self stubPackageInfo extensionMethods do: [ :method | method methodClass removeSelector: method selector ].
	self assert: self stubPackageInfo classes isEmpty.
	self assert: self stubPackageInfo extensionMethods isEmpty.

	package := FLPackageLoader new loadFrom: aStream reset.
	classFactory createdClasses: package classes! !

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
streams
	^ streams! !

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
streams: anObject
	streams := anObject! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 3/26/2011 04:12'!
stubPackageInfo
	^ PackageInfo named: classFactory packageName! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 02:57'!
testClassInitialization
	"It should send #initialize to each class."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass class 
		compileSilently: 'initialize  ClassVariable := 1';
		compileSilently: 'isInitialized  ^ClassVariable isNil not'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.
	
	self assert: materializedClass isInitialized.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 07:14'!
testClassRegistration
	"The class should be included in Smalltalk system dictionary."

	| aClass name materializedClass |
	aClass := classFactory newClass.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.

	materializedClass := 
		Smalltalk 
			at: name 
			ifAbsent: [ self signalFailure: 'The class should be included in Smalltalk system dictionary.' ].
			
	self assert: classFactory defaultCategory equals: (Smalltalk organization categoryOfElement: materializedClass name).
! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:13'!
testIgnoreClassVariableWithClassSideMethod
	"It should not serialize class variable values."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass class
		compileSilently: 'initializeClassVariable  ClassVariable := 1';
		compileSilently: 'classVariable  ^ClassVariable'.
	aClass initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:12'!
testIgnoreClassVariableWithInstanceSideMethod
	"It should not serialize class variable values."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass
		compileSilently: 'initializeClassVariable  ClassVariable := 1';
		compileSilently: 'classVariable  ^ClassVariable'.
	aClass new initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass new classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:14'!
testIgnoreForeignClassSideMethod
	"It should not serialize class foreign protocol."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass class
		compileSilently: 'foreign' classified: '*foreigns';
		compileSilently: 'core' classified: 'cores'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: #(core) asSet equals: materializedClass class localSelectors.	
	self assert: (materializedClass class includesSelector: #core).
	self deny: (materializedClass class includesSelector: #foreign).
	self assert: #(cores) equals: materializedClass class organization categories.
	self assert: #(core) equals: materializedClass class organization allMethodSelectors.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:14'!
testIgnoreForeignInstanceSideMethod
	"It should not serialize class foreign protocol."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass
		compileSilently: 'foreign' classified: '*foreigns';
		compileSilently: 'core' classified: 'cores'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: #(core) asSet equals: materializedClass localSelectors.	
	self assert: (materializedClass includesSelector: #core).
	self deny: (materializedClass includesSelector: #foreign).
	self assert: #(cores) equals: materializedClass organization categories.
	self assert: #(core) equals: materializedClass organization allMethodSelectors.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/21/2011 19:01'!
testPackageWithMethodExtension
	"It should serialize package's extension methods."

	| classWithExtension categoryForMethodExtension |
	categoryForMethodExtension := self stubPackageInfo methodCategoryPrefix.
	classWithExtension := FLPair.
	classWithExtension compileSilently: 'fortyTwo ^42' classified: categoryForMethodExtension. 
	
	self storeRemoveAndLoadStubPackage.
	
	self assert: (classWithExtension includesSelector: #fortyTwo).
	self assert: categoryForMethodExtension equals: (classWithExtension whichCategoryIncludesSelector: #fortyTwo).
	self assert: (PackageOrganizer default packageNames includes: classFactory packageName)! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/21/2011 22:04'!
testPackageWithTwoCategories
	"It should serialize multiple categories by package."

	| aStream classNames |
	classFactory newClassInCategory: 'One'.
	classFactory newClassInCategory: 'Two'.
	classNames := classFactory createdClassNames.

	self storeRemoveAndLoadStubPackage.
	
	self assert: (PackageOrganizer default packageNames includes: classFactory packageName) description: 'The package should be present in PackageOrganizer default.'.! !

Object subclass: #FLMethodExtension
	instanceVariableNames: 'method category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLMethodExtension classSide methodsFor: 'instance creation' stamp: 'MartinDias 3/26/2011 04:41'!
with: aCompiledMethod

	^ self basicNew
		initialize;
		method: aCompiledMethod;
		category: aCompiledMethod category;
		yourself ! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
category
	^ category! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
category: anObject
	category := anObject! !

!FLMethodExtension methodsFor: 'loading' stamp: 'MartinDias 3/26/2011 04:45'!
load

	method methodClass addSelectorSilently: method selector withMethod: method.
	method methodClass organization classify: method selector under: category.! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
method
	^ method! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
method: anObject
	method := anObject! !

Object subclass: #FLPackage
	instanceVariableNames: 'extensionMethods classes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackage classSide methodsFor: 'private' stamp: 'MartinDias 8/22/2011 02:53'!
classesLocalTo: packageInfo
	^ packageInfo classes! !

!FLPackage classSide methodsFor: 'private' stamp: 'MartinDias 8/21/2011 19:21'!
extensionMethodsTo: packageInfo
	^ packageInfo extensionMethods 
		collect: [ :reference | FLMethodExtension with: reference compiledMethod ]! !

!FLPackage classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/21/2011 19:20'!
named: packageName

	| packageInfo |
	packageInfo := PackageInfo named: packageName.
	^ self basicNew
		initialize;
		classes: (self classesLocalTo: packageInfo);
		extensionMethods: (self extensionMethodsTo: packageInfo);
		yourself! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 19:10'!
classes
	^ classes! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 19:10'!
classes: anObject
	classes := anObject! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:35'!
extensionMethods
	^ extensionMethods! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:35'!
extensionMethods: anObject
	extensionMethods := anObject! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 8/22/2011 02:58'!
initializeClasses
	
	self classes do: [ :aClass | aClass initialize ]! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 8/22/2011 07:02'!
load

	self registerInSystemDictionary.
	self registerInSystemOrganization.
	self loadExtensionMethods.
	self initializeClasses.
! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 3/26/2011 04:44'!
loadExtensionMethods
	
	extensionMethods do: [ :extensionMethod | extensionMethod load ]! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 8/21/2011 19:11'!
registerInSystemDictionary

	self classes do: [:aClass | Smalltalk at: aClass name put: aClass ].! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 8/22/2011 07:02'!
registerInSystemOrganization

	self classes do: [:aClass | Smalltalk globals organization classify: aClass name under: aClass category]! !

Object subclass: #FLPackageLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageLoader methodsFor: 'loading' stamp: 'MartinDias 3/26/2011 03:43'!
loadFrom: aStream 

	| package |
	package := (FLMaterializer on: aStream) materialize; root.
	package load.
	^ package.! !

Object subclass: #FLPackageStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageStore methodsFor: 'storing' stamp: 'MartinDias 8/21/2011 19:10'!
storeOn: aStream packageNamed: name 

	| package |
	package := FLPackage named: name.
	(FLSerializer on: aStream)
		analyzer: (FLAnalyzer newFull internalObjects: package classes);
		serialize: package.! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 02:42'!
foreignCategories
	
	^self organization categories select: [ :aCategory | aCategory beginsWith: '*' ]! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:50'!
removeCategorySilently: aCategory

	(organization listAtCategoryNamed: aCategory) do: [ :aSelector | self removeSelectorSilently: aSelector ].
	organization removeCategorySilently: aCategory.
! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 03:05'!
removeForeignProtocol

	self foreignCategories do: [ :aCategory |
		self removeCategorySilently: aCategory ]! !
